# Notes de version Canopsis 24.04.3

Canopsis 24.04.3 a été publié le 11 octobre 2024.

## Procédure d'installation

Suivre la [procédure d'installation de Canopsis](../guide-administration/installation/index.md).

## Procédure de mise à jour

Suivre la [procédure standard de mise à jour de Canopsis](../guide-administration/mise-a-jour/index.md).

### Changements importants

!!! warning "Action type Comportement périodique"

    Dans une étape de scénario de type "Comportement périodique", le nom du comportement doit être unique. Le formulaire prévoit donc dans cette version de spécifier un préfixe.
    Le nom final du comportement périodique sera de la forme "{prefix} {entity_id} {start}-{stop}"

!!! warning "Migration manuelle"

    La transformation de la vue SQL perf_data_name en table nécessite d'exécuter un script de migration manuellement.  
    **En installation Docker**
    
    ```sh
    CPS_EDITION=pro docker compose exec timescaledb psql postgresql://cpspostgres:canopsis@timescaledb:5432/canopsis -v ON_ERROR_STOP=1 --echo-errors -c "BEGIN;
    
    DROP INDEX IF EXISTS perf_data_name_name_idx;
    DROP MATERIALIZED VIEW IF EXISTS perf_data_name CASCADE;
    
    CREATE TABLE IF NOT EXISTS perf_data_name
    (
        name       VARCHAR(255) NOT NULL,
        unit       VARCHAR(5)   NOT NULL,
        updated_at TIMESTAMP    NOT NULL
    );
    CREATE UNIQUE INDEX IF NOT EXISTS perf_data_name_name_idx ON perf_data_name (name);
    
    INSERT INTO perf_data_name (name, unit, updated_at)
    SELECT name, unit, MAX(time)
    FROM perf_data
    GROUP BY name, unit;
    
    END;"
    ```
     
    **En installation RPM**
    
    ```sh
    psql $CPS_POSTGRES_URL -v ON_ERROR_STOP=1 --echo-errors -f /opt/canopsis/share/database/postgres_migrations/17_update_perf_data_name_table.up.sql
    ``` 
     
    Le résultat renvoyé doit être le suivant :  
     
    ```
    BEGIN
    DROP INDEX
    DROP MATERIALIZED VIEW
    CREATE TABLE
    CREATE INDEX
    INSERT 0 0
    COMMIT
    ```


### Corrections de bugs

*  **Interface graphique :**
    * **Comportements périodiques :**
        * Correction d'un bug qui empêchait la création d'un comportement périodique de type "pause" sans fin (#5600)
    * **Bac à alarmes :**
        * Correction d'un bug qui faisait apparaitre une barre de défilement inutilement (#5598)
        * Les "consignes associées" sont à présent incluses dans l'export csv (#5530)
        * Correction d'un bug qui rendait inopérante l'interpétation du code HTML dans la timeline des alarmes (#5625)
*  **Moteurs :**
    * Correction d'un bug qui entrainait une erreur de calcul de durée active/inactive d'alarmes dans certaines situations (#5602, #5619)
    * **Fifo :**
        * Correction d'un bug qui provoquait l'erreur suivante dans le moteur fifo `cannot send tech metrics ERROR: COPY from stdin failed: unable to encode into binary format for int4 ... is greater than maximum value for int4`
    * **Action :**
        * Correction d'un bug qui empêchait les actions scénario de type "comportement périodique" d'être exécutées plusieurs fois (#5599) 
    * **Corrélation :**
        * Correction d'un bug qui permettait à des alarmes d'être attachées à des méta alarmes déjà cloturées (#5573)
   *  **SNMP :**
        * Ajout d'une dépendance python nécessaire à l'installation de Canopsis sur environnement EL9 (#5551)

### Améliorations

*  **Général :**
    * Ajout d'un mécanisme de nommage unique d'une instance Canopsis. Ce nom est visible dans le tooltip du logo Canopsis (#5604)
    * Le testeur de trap "traptester.py" affiche désormais l'événement traduit au lieu de l'événement brut type trap (#5613)
    * La vue SQL perf_data_name a été remplacée par une table en vue d'améliorer des temps de calculs très consommateurs (#5494)
    * Correction d'une erreur de définition Swagger pour la route GET /api/v4/views (#5517)
*  **Interface graphique :**
    * Correction d'un bug qui laissait penser l'utilisateur que le mode édition du menu des vues était activé à tort (en mode "Barre d'entête") (#5636)
    * Le formulaire de création d'un utilisateur active l'utilisateur par défaut (#4778)
*  **Moteurs :**
    * **CHE :**
        * Un enrichissement par collection mongodb externe peut être défini comme optionnel pour éviter d'incrémenter le compteur d'erreur des règles d'enrichissement (#5596) 
    * **Comportements périodiques :**
        * Les comportements périodiques concernants les méta alarmes sont à présent appliqués dès leur création (#5605)
