# Notes de version Canopsis 3.34.0

Canopsis 3.34.0 a été publié le 18 décembre 2019.

## Procédure de mise à jour

Suivre la [procédure standard de mise à jour de Canopsis](../guide-administration/mise-a-jour/index.md).

### Pro : activation des nouveaux moteurs `engine-webhook` et `engine-dynamic-infos`

!!! note
    Cette procédure n'est pas fonctionnelle sur **Debian** dans un environnement de paquets Canopsis Pro Go : le fichier `/opt/canopsis/etc/initialisation-cat.toml.example` est incorrect en 3.34.0 Debian. Vous pouvez soit remplacer ce fichier [par la bonne version](https://git.canopsis.net/canopsis/go-engines/blob/3.34.0/cmd/init/initialisation-cat.toml.example), soit passer directement à la [version 3.35.0](3.35.0.md) qui corrige cela.

Cette partie ne concerne que les installations Go Pro.

**En installation par paquets Pro Go**, après avoir réalisé la mise à jour, vous devez exécuter les commandes suivantes pour activer les deux nouveaux moteurs Pro :

```sh
# Arrêt de Canopsis (cause une interruption de service)
canoctl stop

# Ici, supprimer tout appel à « -postProcessorsDirectory /opt/canopsis/lib/go/plugins/axepostprocessor »
# si celui-ci était présent dans la configuration systemd du moteur axe.
vi $(grep -lr "postProcessorsDirectory" /etc/systemd/system/canopsis*)

# Reroutage du moteur watcher pour qu'il renvoie ses données à dynamic-infos
mkdir -p /etc/systemd/system/canopsis-engine-go@engine-watcher.service.d
cat > /etc/systemd/system/canopsis-engine-go@engine-watcher.service.d/watcher.conf << EOF
[Service]
ExecStart=
ExecStart=/usr/bin/env /opt/canopsis/bin/%i -publishQueue Engine_dynamic_infos
EOF

# Reroutage du moteur dynamic-infos pour qu'il renvoie ses données à webhook
mkdir -p /etc/systemd/system/canopsis-engine-go@engine-dynamic-infos.service.d
cat > /etc/systemd/system/canopsis-engine-go@engine-dynamic-infos.service.d/dynamic-infos.conf << EOF
[Service]
ExecStart=
ExecStart=/usr/bin/env /opt/canopsis/bin/%i -publishQueue Engine_webhook
EOF

# Reroutage du moteur webhook pour qu'il renvoie ses données à action
mkdir -p /etc/systemd/system/canopsis-engine-go@engine-webhook.service.d
cat > /etc/systemd/system/canopsis-engine-go@engine-webhook.service.d/webhook.conf << EOF
[Service]
ExecStart=
ExecStart=/usr/bin/env /opt/canopsis/bin/%i -publishQueue Engine_action
EOF

# Charger les modifications
systemctl daemon-reload

# On s'assure que les moteurs action et watcher étaient bien activés
systemctl enable canopsis-engine-go@engine-action
systemctl enable canopsis-engine-go@engine-watcher

# Activation des nouveaux moteurs
systemctl enable canopsis-engine-go@engine-dynamic-infos
systemctl enable canopsis-engine-go@engine-webhook

# Création des files RabbitMQ dédiées à ces nouveaux moteurs
cp /opt/canopsis/etc/initialisation-cat.toml.example /opt/canopsis/etc/initialisation.toml
set -o allexport ; source /opt/canopsis/etc/go-engines-vars.conf
/opt/canopsis/bin/init -conf /opt/canopsis/etc/initialisation.toml

# Démarrage de Canopsis avec la nouvelle configuration
canoctl start
```

**En installation Docker Pro Go**, supprimez d'abord tout paramètre `-postProcessorsDirectory /plugins/axepostprocessor` qui pourrait être utilisé, et remplacez tout appel à l'image `canopsis/engine-axe-cat` par `canopsis/engine-axe`.

Puis, assurez-vous que les moteurs suivants utilisent bien les arguments `-publishQueue` suivants :

```yaml
  watcher:
    image: canopsis/engine-watcher:${CANOPSIS_IMAGE_TAG}
    env_file:
      - compose.env
    restart: unless-stopped
    command: /engine-watcher -publishQueue Engine_dynamic_infos
  dynamic-infos:
    image: canopsis/engine-dynamic-infos:${CANOPSIS_IMAGE_TAG}
    env_file:
      - compose.env
    depends_on:
      - "mongodb"
    restart: unless-stopped
    command: /cmd/engine-dynamic-infos -publishQueue Engine_webhook
  webhook:
    image: canopsis/engine-webhook:${CANOPSIS_IMAGE_TAG}
    env_file:
      - compose.env
    depends_on:
      - "mongodb"
    restart: unless-stopped
    command: /cmd/engine-webhook -publishQueue Engine_action
```

En attendant l'arrivée d'une nouvelle image `init-cat` dans une prochaine version ([Canopsis 3.36.0](3.36.0.md)), récupérez [le fichier `initialisation-cat.toml`](https://git.canopsis.net/canopsis/go-engines/blob/develop/cmd/init/initialisation-cat.toml.example) et assurez-vous qu'il soit utilisé par l'image `init` à l'aide d'un volume :

```yaml
  init:
    image: canopsis/init:${CANOPSIS_IMAGE_TAG}
    depends_on:
      - "rabbitmq"
      - "mongodb"
    env_file:
      - compose.env
    volumes:
      - "./files/initialisation-cat.toml.example:/initialisation.toml"
```

Puis, terminer le chargement de ces modification avec un `docker-compose up -d`.

## Changements entre Canopsis 3.33.0 et 3.34.0

Voir le [ChangeLog complet](https://git.canopsis.net/canopsis/canopsis/blob/develop/CHANGELOG.md) pour plus de détails techniques.

*  **Documentation :**
    *  Ajout d'une documentation sur l'API service weather ([!1255](https://git.canopsis.net/canopsis/canopsis/merge_requests/1255))
    *  Ajout de [documentation sur le nouveau moteur `engine-dynamic-infos` - lien obsolète]() ainsi qu'un [cas d'usage associé](../guide-utilisation/cas-d-usage/affichage-de-consignes.md) ([!1263](https://git.canopsis.net/canopsis/canopsis/merge_requests/1263))
*  **Moteurs :**
    *  **Moteurs nouvelle génération (Go) :**
        *  Pro : Suppression du plugin `webhook.so` et ajout d'un nouveau moteur `engine-webhook` disposant des mêmes fonctionnalités. Ceci est nécessaire pour le développement de ses futures fonctionnalités ([#2030](https://git.canopsis.net/canopsis/canopsis/issues/2030))
        *  Pro : Ajout d'un nouveau moteur `engine-dynamic-infos`, permettant d'enrichir dynamiquement des alarmes ([#2056](https://git.canopsis.net/canopsis/canopsis/issues/2056))
*  **UI :**
    *  Ajout d'un mécanisme de suggestions dans les éditeurs de filtres ([#2059](https://git.canopsis.net/canopsis/canopsis/issues/2059))
    *  Correction des boutons d'ajout dans le formulaire de création d'entités ([#2060](https://git.canopsis.net/canopsis/canopsis/issues/2060))
    *  **Bac à alarmes :**
        *  Ajout d'une action de masse d'association de tickets sur les alarmes ([#2055](https://git.canopsis.net/canopsis/canopsis/issues/2055))
        *  Correction de l'évènement envoyé lors d'un clic sur le bouton « Acquitter et déclarer un ticket » ([#2051](https://git.canopsis.net/canopsis/canopsis/issues/2051))
    *  **Explorateur de contexte :**
        *  Correction des colonnes `enabled` dans l'Explorateur de contexte ([#2049](https://git.canopsis.net/canopsis/canopsis/issues/2049))
    *  **CRUD Informations dynamiques :**
        *  Pro : Ajout d'une interface pour la gestion des Informations dynamiques apportées par le nouveau moteur `engine-dynamic-infos` ([#2058](https://git.canopsis.net/canopsis/canopsis/issues/2058))
    *  **CRUD pbehavior :**
        *  Correction d'une régression dans les filtres du formulaire d'ajout d'un *pbehavior* ([#2062](https://git.canopsis.net/canopsis/canopsis/issues/2062))
