# Notes de version Canopsis 4.6.0

Canopsis 4.6.0 a été publié le 29 juillet 2022.

## Procédure d'installation

Suivre la [procédure d'installation de Canopsis](../guide-administration/installation/index.md).

## Procédure de mise à jour

Canopsis 4.6.0 apporte des changements importants tant au niveau technique que fonctionnel. À ce titre, le [Guide de migration vers Canopsis 4.6.0](migration/migration-4.6.0.md) doit obligatoirement être suivi pour les mises à jour d'installations déjà en place.


## Changements entre Canopsis 4.5.x et 4.6.0


### Ports d'écoute du reverse proxy Nginx

Pour des raisons de sécurité, le reverse proxy `nginx` fourni avec Canopsis dans les images Docker n'écoute plus les ports **80** et **443**.  
A présent, le port 8080 est utilisé pour l'écoute http et le 8443 pour l'écoute https.  
Consultez le [guide de migration](migration/migration-4.6.0.md) pour adapter vos configurations.

### Options `-enrichContext` et `-enrichInclude` obsolètes

Ces options du moteur `engine-che` sont à présent obsolètes et ne peuvent plus être utilisées.  
L'équivalent fonctionnel est à effectuer par l'intermédiaire de règles d'enrichissement dans le moteur.

Consultez le [guide de migration](migration/migration-4.6.0.md) pour adapter vos configurations.


### Vues et groupes de vues

Le format interne des vues a été complètement repensé.  
Un impact est à prévoir sur les éléments suivants : 

* Les vues
* Les groupes de vues ainsi que le menu des vues
* La fonction è import/export` de vues/groupes

La migration s'effectue grâce à la procédure mentionnée dans le [guide de migration](migration/migration-4.6.0.md) et particulièrement grâce au script `001_migrate_views.js`.

### Zerolog

Il est désormais possible de configurer le `logger` de Canopsis grâce aux options suivantes du fichier `canopsis.toml`

**Section [Canopsis.logger]**

| Attribut            | Exemple de valeur  | Description                                             |
| :------------------ | :------------------| :------------------------------------------------------ |
| Writer              | "stdout"           | Canal de sortie du logger. **`stdout`** ou **`stderr`** |

**Sous-section [Canopsis.logger.console_writer]**

| Attribut            | Exemple de valeur                           | Description                                             |
| :------------------ | :-------------------------------------------| :------------------------------------------------------ |
| Enabled             | true                                        | Active ou désactive le mode [ConsoleWriter](https://github.com/rs/zerolog#pretty-logging). Si désactivé alors les messages sont loggués en JSON. |
| NoColor             | true                                        | Active ou désactive les couleurs dans les logs |
| TimeFormat          | "2006-01-02T15:04:05Z07:00"                 | Format des dates des messages de logs au format GO |
| PartsOrder          | ["time", "level", "caller", "message"]      | Ordre des parties des messages de logs parmi "time", "level", "message", "caller", "error" |


### Surcharge de la configuration canopsis.toml

Le fichier de configuration `canopsis.toml` peut être surchargé par un autre fichier défini grâce à l'option `-override` de la commande `canopsis-reconfigure`.

Pour rappel, l'emplacement du fichier de configuration diffère entre les différents types d'environnement d'installation proposés par Canopsis.

| Type d'environnement | Emplacement du fichier            |
|----------------------|-----------------------------------|
| Paquets RPM              | `/opt/canopsis/etc/canopsis.toml` |
| Docker Compose Pro       | `/canopsis-pro.toml` dans le conteneur `reconfigure` |
| Docker Compose Community | `/canopsis-community.toml` dans le conteneur `reconfigure` |


### Nouvelle version de l'outil `amqp2tty`

L'outil `amqp2tty` a été remplacé par une version en GO.  
Son utilisation est similaire à celle de la version python.  

=== "Paquets CentOS 7"

    ```sh
	set -o allexport ; source /opt/canopsis/etc/go-engines-vars.conf
	/opt/canopsis/bin/amqp2tty 
    ```

=== "Docker Compose"

    ```sh
	docker run --env CPS_AMQP_URL=amqp://cpsrabbit:canopsis@rabbitmq/canopsis \
	--network=canopsis-pro_default \
	docker.canopsis.net/docker/community/amqp2tty:4.6.0
    ```

### Connecteur Zabbix Webhook

Un nouveau connecteur événementiel Zabbix vers Canopsis a été publié.  
Contrairement au précédent, celui-ci s'appuie sur le mécanisme de Webhook de Zabbix.  
Les versions compatibles de Zabbix sont 5.0 et supérieure.

Vous pouvez retrouver la [documentation ici](../interconnexions/Supervision/Zabbix.md)

### Suppression des widgets obsolètes

Les widgets suivants ont été supprimés de Canopsis.   : `Statshistogram`, `Stats curves`, `Stats table`, `Stats number`, `Pareto diagram`.
Ils ne sont désormais plus utilisables.

![Useless Widgets](./img/useless-widgets.png)

### Liste des modifications

*  **UI :**
    * Possibilité de charger un fichier javascript externe (#3912)
    * Suppression des widgets obsolètes`Statshistogram`, `Stats curves`, `Stats table`, `Stats number`, `Pareto diagram` (#4116)
    * Remplacement du mot `Instruction` par `Consigne` dans les menus de remédiation (#4276)
    * Le dernier commentaire d'une alarme est disponible dans les templates GO via `{{ .Alarm.Value.LastComment.Message }}` (#4416)
    * La fonction "Lister les comportements périodiques" permet maintenant d'afficher tous les comportements sur un même calendrier (#132)
    * De manière générale, les popup ne sont plus au format plein écran (#4254)
    * **Météo des Services**
        * Correction d'un bug qui permettait d'effectuer une boucle de popup (#4381)
        * L'affichage de la priorité est maintenant optionnelle (#4396)
        * Les actions de masse sont désormais disponibles depuis les modales de météo (#133)
    * **Bac à alarmes**
        * Il est maintenant possible d'attribuer un style aux contenus des colonnes (#4065)
        * Ajout d'un mode `compact` qui permet de présenter une quantité d'informations plus importante dans le même espace (#4251)
        * Dans un bac à alarme, la priorité est donnée à la recherche vis-à-vis du rafraîchissement périodique (#3732)
        * A moins de débrider l'option, il n'est plus possible de déclarer plusieurs tickets sur une alarme (#4323)
        * Un nouveau jeu d'icônes est mis en oeuvre pour les différents statuts de remédiation automatiques (#4057)
    * **Explorateur de contexte**
        * Les entités peuvent être activées ou désactivées par une action de masse (#4279)
    * **Administration**
        * Correction d'un bug qui empêchait la validation d'un template valide de job de remédiation (#4383)
        * Les messages de diffusion supportent à présent le format HTML (#4115)
    * **Exploitation**
        * Les comportements périodiques peuvent être activés ou désactivés en masse (#4278)
*  **API :**
    * L'intervalle de mise à jour des informations `Healtchcheck` est maintenant paramétrable (#4142)
*  **Moteurs :**
    * **Axe**
        * Lorsqu'une alarme atteint 2000 steps, elle accepte désormais un retour à l'état 0 ou une annulation (#4368)
    * **Remediation**
        * De nouveaux triggers sont disponibles : `instructionfail`, `autoinstructionfail`, `instructionjobfail`, `instructioncomplete`, `autoinstructioncomplete`. [Voir doc](https://doc.canopsis.net/guide-administration/architecture-interne/triggers/) (#4294)
    * **Webhook**
        * Ajout de la variable `.ResponseMap` qui stocke l'ensemble des réponses des webhooks (#4340)
*  **Général :**
    * AMQP : Le driver AMQP `streadway` a été changé au profit du driver officiel RabbitMQ `amqp091-go` (#4253)
    * Il est désormais possible de surcharger le fichier de configuration `canopsis.toml` (#118)
    * Timescaledb est maintenant intégré au module `Healtchcheck` (#4238)
*  **Documentation :**
    * [Documentation des triggers](https://doc.canopsis.net/guide-administration/architecture-interne/triggers/), swagger mis à jour en conséquence (#4294, #4044)
    * [Scenarios](https://doc.canopsis.net/guide-utilisation/menu-exploitation/scenarios/) (#4288)
    * [Zerolog](https://doc.canopsis.net/guide-administration/administration-avancee/modification-canopsis-toml/) (#3761)
    * [Surcharge du fichier canopsis.toml](https://doc.canopsis.net/guide-administration/administration-avancee/modification-canopsis-toml/) (#118)
    * [Dernier commentaire](https://doc.canopsis.net/guide-utilisation/templates-go/) (#4416)
    * [.ResponseMap](https://doc.canopsis.net/guide-utilisation/menu-exploitation/scenarios/) (#4340)
    * [Widget Junit](https://doc.canopsis.net/guide-utilisation/interface/widgets/junit) (#215)
    * [AMQP2TTY, version GO](https://doc.canopsis.net/guide-de-depannage/amqp2tty/) (#4312)
